<?php

namespace backend\controllers;

use common\forms\Company\ClientForm;
use common\forms\Company\CompanyForm;
use common\readModels\CompanyReadRepository;
use common\readModels\ContractReadRepository;
use common\services\CompanyService;
use DomainException;
use Yii;
use common\entities\Company\Company;
use yii\base\Module;
use yii\web\NotFoundHttpException;
use yii\web\Response;

/**
 * CompanyController реализует CRUD операции над юридическими лицами.
 */
class CompanyController extends CrudController
{
    private CompanyService $companyService;
    private CompanyReadRepository $companyReadRepository;

    /** @param array<string> $config */
    public function __construct(
        string $id,
        Module $module,
        CompanyService $service,
        CompanyReadRepository $companyReadRepository,
        array $config = []
    ) {
        parent::__construct($id, $module, $config);

        $this->companyService = $service;
        $this->companyReadRepository = $companyReadRepository;
    }

    /**
     * Список всех юридических лиц.
     */
    public function actionIndex(): string
    {
        $dataProvider = $this->companyReadRepository->getAllCompanies();

        return $this->render('index', [
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Список клиентов.
     * @return string|Response
     */
    public function actionClients()
    {
        $dataProvider = $this->companyReadRepository->getAllClients();

        $form = new ClientForm();

        if ($form->load(Yii::$app->request->post()) && $form->validate()) {
            try {
                $this->companyService->makeClient($form);
                return $this->redirect(['clients']);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('clients', [
            'dataProvider' => $dataProvider,
            'model' => $form,
        ]);
    }

    /**
     * Рендер карточки юридического лица.
     * @throws NotFoundHttpException если модель не найдена
     */
    public function actionView(int $id): string
    {
        $contracts = new ContractReadRepository();
        $contractsProvider = $contracts->getContractsByCompany($id);

        return $this->render('view', [
            'model' => $this->findModel($id),
            'contractsProvider' => $contractsProvider,
        ]);
    }

    /**
     * Добавить запись юридического лица.
     * Перенаправление на карточку после создания.
     * @return string|Response
     */
    public function actionCreate()
    {
        $form = new CompanyForm();

        if ($form->load(Yii::$app->request->post()) && $form->validate()) {
            try {
                $company = $this->companyService->create($form);
                return $this->redirect(['view', 'id' => $company->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('create', [
            'model' => $form,
        ]);
    }

    /**
     * Обновить запись юридического лица.
     * Перенаправление на карточку после обновления.
     * @return string|Response
     * @throws NotFoundHttpException если модель не найдена
     */
    public function actionUpdate(int $id)
    {
        $company = $this->findModel($id);
        $form = new CompanyForm($company);

        if ($form->load(Yii::$app->request->post()) && $form->validate()) {
            try {
                $this->companyService->edit($company->id, $form);
                return $this->redirect(['view', 'id' => $company->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('update', [
            'model' => $form,
            'company' => $company,
        ]);
    }

    /**
     * Удалить запись юридического лица.
     * Перенаправление на список юридических лиц после удаления.
     */
    public function actionDelete(int $id): Response
    {
        try {
            $this->companyService->remove($id);
        } catch (DomainException $e) {
            Yii::$app->errorHandler->logException($e);
            Yii::$app->session->setFlash('error', $e->getMessage());
        }

        return $this->redirect(['index']);
    }

    /**
     * Поиск юридического лица по ID.
     * @return Company the loaded model
     * @throws NotFoundHttpException если модель не найдена
     */
    protected function findModel(int $id): Company
    {
        $company = $this->companyReadRepository->find($id);
        if (($company !== null)) {
            return $company;
        }

        throw new NotFoundHttpException('Страницы не существует.');
    }
}
