<?php

namespace backend\controllers;

use common\entities\Project\PdfDoesNotExist;
use common\entities\Project\PdfPageDoesNotExist;
use common\entities\Project\PdfParser;
use common\entities\Project\ProjectImage;
use common\entities\Project\ProjectWorkQuery;
use common\entities\Project\ProjectWorksContractor;
use common\forms\ProjectUploadPdfForm;
use DomainException;
use common\helpers\ProjectHelper;
use ImagickException;
use Yii;
use yii\base\ErrorException;
use yii\base\Exception;
use yii\data\ActiveDataProvider;
use yii\helpers\FileHelper;
use yii\web\NotFoundHttpException;
use yii\web\Response;
use yii\web\UploadedFile;

/**
 * ProjectController реализует операции над проектными документами.
 */
class ProjectController extends CrudController
{
    /**
     * Lists all ProjectWorksContractor models per contract_id.
     */
    public function actionIndex(int $contractId): string
    {
        $query = new ProjectWorkQuery();
        $query = $query->getWorksQuery($contractId);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);

        return $this->render('index', [
            'dataProvider' => $dataProvider,
            'contractId' => $contractId,
        ]);
    }

    /**
     * Lists all ProjectImage models per contract_id.
     */
    public function actionPages(int $contractId, $workTypeId): string
    {
        $query = new ProjectWorkQuery();
        $query = $query->getPagesQuery($contractId, $workTypeId);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);

        return $this->render('pages', [
            'dataProvider' => $dataProvider,
            'contractId' => $contractId,
        ]);
    }

    /**
     * @throws ImagickException
     * @throws PdfDoesNotExist
     * @throws Exception
     * @throws PdfPageDoesNotExist
     * @return Response|array
     */
    public function actionSaveImage()
    {
        $data = Yii::$app->request->post();
        if (Yii::$app->request->isAjax && $data) {
            Yii::$app->response->format = Response::FORMAT_JSON;
            $pdfName = basename($data['completePath']);

            try {
                $pdf = new PdfParser($data['completePath'], $data['pageCount']);
                $pdf->setPage($data['page']);
                //$pdf->setResolution(72);
                $page = $pdf->saveImage(ProjectHelper::getImageSourcePath($pdfName));

                if ($page && $data['contractId']) {
                    ProjectImage::create($page, $data['contractId'], $pdfName);
                }

                return ['result' => true, 'pageCount' => $data['pageCount'], 'completePath' => $data['completePath'], 'contractId' => $data['contractId']];
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }

            return ['result' => false, 'error' => $e->getMessage()];
        }

        return $this->redirect(['upload-pdf', 'contractId' => $data['contractId']]);
    }

    /**
     * @throws PdfDoesNotExist
     * @throws Exception
     * @throws ImagickException
     * @return string|array
     */
    public function actionUploadPdf(int $contractId)
    {
        $model = new ProjectUploadPdfForm();

        if (Yii::$app->request->isPost) {
            $model->file = UploadedFile::getInstance($model, 'file');

            if ($model->file && $model->validate()) {
                Yii::$app->response->format = Response::FORMAT_JSON;
                try {
                    $pdfName = $model->savePdf($model->file);

                    $completePath = ProjectHelper::getPdfSourcePath() . '/' . $pdfName;
                    $pdf = new PdfParser($completePath);
                    $pageCount = $pdf->getNumberOfPages();

                    return ['result' => true, 'pageCount' => $pageCount, 'completePath' => $completePath];
                } catch (DomainException $e) {
                    Yii::$app->errorHandler->logException($e);
                    Yii::$app->session->setFlash('error', $e->getMessage());
                }
            }
        }

        return $this->render('upload_pdf', [
            'model' => $model,
            'contractId' => $contractId,
        ]);
    }

    /**
     * @throws Exception
     * @return string|Response
     * @throws ErrorException
     */
    public function actionImageWorkType(int $contractId)
    {
        $model = new ProjectImage();

        $images = Yii::$app->request->post('ProjectImage', []);

        if (Yii::$app->getRequest()->isPost && $images) {
            try {
                $projectPages = ProjectImage::edit($images);
                ProjectWorksContractor::edit($projectPages);

                return $this->redirect(['index', 'contractId' => $contractId]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        FileHelper::removeDirectory(ProjectHelper::getPdfSourcePath());

        return $this->render('images_types', [
            'model' => $model,
            'contractId' => $contractId,
        ]);
    }

    /**
     * @throws NotFoundHttpException
     * @throws Exception
     */
    public function actionShowPdf(int $id): Response
    {
        $model = $this->findImageModel($id);

        $completePath = ProjectHelper::getPdfSourcePath() . '/' . $model->pdf_name;

        return Yii::$app->response->sendFile($completePath, $model->pdf_name, ['mimeType' => 'application/pdf', 'inline' => true]);
    }

    /**
     * @throws NotFoundHttpException
     * @throws Exception
     */
    public function actionShowImage(int $id): void
    {
        $image = $this->findImageModel($id);

        $response = Yii::$app->getResponse();
        $response->headers->set('Content-Type', 'image/jpeg');
        $response->format = Response::FORMAT_RAW;

        $completePath = ProjectHelper::getImageSourcePath($image->pdf_name) . '/' . $image->name;
        $response->stream = fopen($completePath, 'r');

        $response->send();
    }

    /**
     * Updates an existing ContractExpense model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @throws NotFoundHttpException if the model cannot be found
     * @return string|Response
     */
    public function actionContractorUpdate(int $id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['project/index', 'contractId' => $model->contract_id]);
        }

        return $this->render('update_contractor', [
            'model' => $model,
        ]);
    }

    /**
     * Finds the ProjectWorksContractor model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @return ProjectWorksContractor the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel(int $id): ProjectWorksContractor
    {
        if (($model = ProjectWorksContractor::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('Запись не найдена.');
    }

    /**
     * Finds the ProjectImage model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @throws NotFoundHttpException if the model cannot be found
     * @return ProjectImage the loaded model
     */
    protected function findImageModel(int $id): ProjectImage
    {
        if (($model = ProjectImage::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('Запись не найдена.');
    }
}
