<?php

namespace backend\controllers\contract;

use backend\controllers\CrudController;
use common\entities\Contract\ContractCompany;
use common\entities\Contract\ContractExpense;
use common\entities\Contract\ContractWork;
use common\forms\Contract\ContractAmountForm;
use common\forms\Contract\ContractForm;
use common\readModels\ContractReadRepository;
use common\services\ContractCompanyService;
use common\services\ContractService;
use DomainException;
use Yii;
use common\entities\Contract\Contract;
use yii\base\Model;
use yii\base\Module;
use yii\bootstrap4\ActiveForm;
use yii\data\ActiveDataProvider;
use yii\web\NotFoundHttpException;
use yii\web\Response;

/**
 * ContractController реализует CRUD операции над договорами.
 */
class ContractController extends CrudController
{
    private ContractService $service;
    private ContractCompanyService $contractCompanyService;
    private ContractReadRepository $contract;

    /** @param array<string> $config */
    public function __construct(
        string $id,
        Module $module,
        ContractService $service,
        ContractCompanyService $contractCompanyService,
        ContractReadRepository $contract,
        array $config = []
    ) {
        parent::__construct($id, $module, $config);

        $this->service = $service;
        $this->contractCompanyService = $contractCompanyService;
        $this->contract = $contract;
    }

    public function actions(): array
    {
        return [
            'history' => [
                'class' => 'common\modules\entitiesHistory\actions\IndexAction',
                'modelClass' => 'common\entities\Contract\Contract',
                'viewFile' => 'index_history',
                'pageSize' => 20,
            ],
            'history-delete' => [
                'class' => 'common\modules\entitiesHistory\actions\DeleteAction',
                'redirectUrl' => ['contract/view'],
            ],
        ];
    }

    /**
     * Lists all Contract models.
     */
    public function actionIndex(int $companyId): string
    {
        $dataProvider = $this->contract->getContractsByCompany($companyId);

        return $this->render('index', [
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Рендер карточки договора.
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView(int $id): string
    {
        $workProvider = new ActiveDataProvider([
            'query' => ContractWork::find()->where(['contract_id' => $id]),
        ]);
        $expenseProvider = new ActiveDataProvider([
            'query' => ContractExpense::find()->where(['contract_id' => $id]),
        ]);

        return $this->render('view', [
            'model' => $this->findModel($id),
            'workProvider' => $workProvider,
            'expenseProvider' => $expenseProvider,
        ]);
    }

    /**
     * Добавить запись договора.
     * Перенаправление на карточку после создания.
     * @return string|Response|array
     */
    public function actionCreate(int $companyId)
    {
        $form = new ContractForm();

        $request = Yii::$app->getRequest();
        $data = Yii::$app->request->post('ContractCompany', []);
        if ($request->isPost && $request->post('ajax') !== null) {
            $result = $this->materialValidation($data);
            Yii::$app->response->format = Response::FORMAT_JSON;
            return $result;
        }

        if ($form->load(Yii::$app->request->post()) && $form->validate()) {
            try {
                $contract = $this->service->create($form);
                $this->contractCompanyService->batchUpdate($data, $contract);
                return $this->redirect(['contract/view', 'id' => $contract->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('create', [
            'model' => $form,
            'companyId' => $companyId,
        ]);
    }

    /**
     * Обновить запись договора.
     * Перенаправление на карточку после обновления.
     * @return string|Response|array
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $contract = $this->findModel($id);
        $form = new ContractForm($contract);

        $request = Yii::$app->getRequest();
        $data = Yii::$app->request->post('ContractCompany', []);
        if ($request->isPost && $request->post('ajax') !== null) {
            $result = $this->materialValidation($data);
            Yii::$app->response->format = Response::FORMAT_JSON;

            return $result;
        }

        if ($form->load(Yii::$app->request->post()) && $form->validate()) {
            try {
                $contract = $this->service->edit($id, $form);
                $this->contractCompanyService->batchUpdate($data, $contract);

                return $this->redirect(['contract/view', 'id' => $contract->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('update', [
            'model' => $form,
            'contract' => $contract,
        ]);
    }

    /**
     * Валидация полей материалов
     * @param array<string|int> $data
     * @return array
     */
    public function materialValidation(array $data): array
    {
        $contractCompanies = [new ContractCompany()];
        foreach (array_keys($data) as $index) {
            $contractCompanies[$index] = new ContractCompany();
        }
        Model::loadMultiple($contractCompanies, Yii::$app->request->post());

        return ActiveForm::validateMultiple($contractCompanies);
    }

    /**
     * Обновить запись договора.
     * Перенаправление на карточку после обновления.
     * @return string|Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdateAmount($id)
    {
        $contract = $this->findModel($id);
        $form = new ContractAmountForm($contract);

        if ($form->load(Yii::$app->request->post()) && $form->validate()) {
            try {
                $contract = $this->service->amountEdit($id, $form);
                return $this->redirect(['view', 'id' => $contract->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('amount_form', [
            'model' => $form,
            'contract' => $contract,
        ]);
    }

    /**
     * Удалить запись договора.
     * Перенаправление на список договоров после удаления.
     */
    public function actionDelete($id): Response
    {
        try {
            $this->service->remove($id);
        } catch (DomainException $e) {
            Yii::$app->errorHandler->logException($e);
            Yii::$app->session->setFlash('error', $e->getMessage());
        }

        return $this->redirect(['index']);
    }

    /**
     * Поиск договора по ID.
     * @return Contract the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel(int $id): Contract
    {
        if ($contract = $this->contract->find($id)) {
            return $contract;
        }

        throw new NotFoundHttpException('Страницы не существует.');
    }
}
