<?php

namespace backend\controllers\contract;

use backend\controllers\CrudController;
use common\forms\Contract\ContractEstimateForm;
use common\readModels\ContractEstimateReadRepository;
use common\services\ContractEstimateService;
use DomainException;
use Throwable;
use Yii;
use common\entities\Contract\ContractEstimate;
use yii\base\Module;
use yii\db\StaleObjectException;
use yii\web\NotFoundHttpException;
use yii\web\Response;

/**
 * ContractEstimateController реализует CRUD операции над сметами по договору.
 */
class ContractEstimateController extends CrudController
{
    private ContractEstimateService $service;
    private ContractEstimateReadRepository $estimateReadRepository;

    /** @param array<string> $config */
    public function __construct(
        string $id,
        Module $module,
        ContractEstimateService $service,
        ContractEstimateReadRepository $estimateReadRepository,
        array $config = []
    ) {
        parent::__construct($id, $module, $config);

        $this->service = $service;
        $this->estimateReadRepository = $estimateReadRepository;
    }

    public function actions(): array
    {
        return [
            'history' => [
                'class' => 'common\modules\entitiesHistory\actions\IndexAction',
                'modelClass' => 'common\entities\Contract\ContractEstimate',
                'viewFile' => 'index_history',
                'pageSize' => 20,
            ],
        ];
    }

    /**
     * Lists all ContractEstimate models.
     */
    public function actionIndex(int $contractId): string
    {
        $dataProvider = $this->estimateReadRepository->getEstimatesByContract($contractId);

        return $this->render('index', [
            'dataProvider' => $dataProvider,
            'contractId' => $contractId,
        ]);
    }

    /**
     * Displays a single ContractEstimate model.
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView(int $id): string
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new ContractEstimate model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return string|Response
     */
    public function actionCreate(int $contractId)
    {
        $form = new ContractEstimateForm();

        if ($form->load(Yii::$app->request->post()) && $form->validate()) {
            try {
                $this->service->create($form);
                return $this->redirect(['contract-estimate/index', 'contractId' => $contractId]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('create', [
            'model' => $form,
            'contractId' => $contractId,
        ]);
    }

    /**
     * Updates an existing ContractEstimate model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @return string|Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate(int $id)
    {
        $estimate = $this->findModel($id);
        $form = new ContractEstimateForm($estimate);

        if ($form->load(Yii::$app->request->post()) && $form->validate()) {
            try {
                $estimate = $this->service->edit($id, $form);
                return $this->redirect(['contract-estimate/index', 'contractId' => $estimate->contract->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('update', [
            'model' => $form,
            'contractId' => $estimate->contract->id,
        ]);
    }

    /**
     * Deletes an existing ContractEstimate model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @throws NotFoundHttpException if the model cannot be found
     * @throws Throwable
     * @throws StaleObjectException
     */
    public function actionDelete(int $id): Response
    {
        $estimate = $this->findModel($id);
        $estimate->delete();

        return $this->redirect(['index', 'contractId' => $estimate->contract->id]);
    }

    /**
     * Finds the ContractEstimate model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @return ContractEstimate the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id): ContractEstimate
    {
        if ($estimateReadRepository = $this->estimateReadRepository->find($id)) {
            return $estimateReadRepository;
        }

        throw new NotFoundHttpException('Страницы не существует.');
    }
}
