<?php

namespace backend\controllers\dictionaries;

use backend\controllers\CrudController;
use common\entities\Dictionaries\ConsumptionRatesMaterial;
use common\services\ConsumptionRateService;
use common\services\ConsumptionRatesMaterialService;
use DomainException;
use Throwable;
use Yii;
use common\entities\Dictionaries\ConsumptionRate;
use yii\base\Model;
use yii\base\Module;
use yii\bootstrap4\ActiveForm;
use yii\data\ActiveDataProvider;
use yii\db\IntegrityException;
use yii\db\StaleObjectException;
use yii\web\ForbiddenHttpException;
use yii\web\NotFoundHttpException;
use yii\web\Response;

/**
 * ConsumptionRateController реализует CRUD операции над нормами расходов.
 */
class ConsumptionRateController extends CrudController
{
    private ConsumptionRatesMaterialService $materialService;
    private ConsumptionRateService $service;

    /** @param array<string> $config */
    public function __construct(
        string $id,
        Module $module,
        ConsumptionRatesMaterialService $materialService,
        ConsumptionRateService $service,
        array $config = []
    ) {
        parent::__construct($id, $module, $config);

        $this->materialService = $materialService;
        $this->service = $service;
    }

    /**
     * Lists all ConsumptionRate models.
     */
    public function actionIndex(): string
    {
        $dataProvider = new ActiveDataProvider([
            'query' => ConsumptionRate::find(),
            'sort' => [
                'defaultOrder' => [
                    'created_at' => SORT_DESC,
                ],
            ],
        ]);

        return $this->render('index', [
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single ConsumptionRate model.
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView(int $id): string
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new ConsumptionRate model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return string|Response
     */
    public function actionCreate()
    {
        $model = new ConsumptionRate();

        if ($model->load(Yii::$app->request->post())) {
            $rate = $this->service->create($model->work_list_id, $model->work_unit_cost, $model->work_scope);
            try {
                $this->materialService->addMaterials($rate->id, $rate->work_list_id);
                return $this->redirect(['update-materials', 'id' => $rate->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing ConsumptionRate model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @return string|Response|array
     * @throws NotFoundHttpException if the model cannot be found
     * @throws Throwable
     */
    public function actionUpdateMaterials(int $id)
    {
        $model = $this->findModel($id);

        $request = Yii::$app->getRequest();
        $data = Yii::$app->request->post('ConsumptionRatesMaterial', []);

        if ($request->isPost && ($request->post('ajax') !== null)) {
            $result = $this->materialValidation($data);
            Yii::$app->response->format = Response::FORMAT_JSON;

            return $result;
        }

        if ($request->isPost && $data) {
            try {
                $this->materialService->batchUpdate($data);

                return $this->redirect(['view', 'id' => $model->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('materials_form', [
            'model' => $model,
        ]);
    }

    /**
     * Валидация полей материалов.
     * @param array<string|int> $data
     * @return array
     */
    public function materialValidation(array $data): array
    {
        $consumptionRateMaterials = [new ConsumptionRatesMaterial()];
        foreach (array_keys($data) as $index) {
            $consumptionRateMaterials[$index] = new ConsumptionRatesMaterial();
        }
        Model::loadMultiple($consumptionRateMaterials, Yii::$app->request->post());

        return ActiveForm::validateMultiple($consumptionRateMaterials);
    }

    /**
     * Updates an existing ConsumptionRate model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @return string|Response
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate(int $id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post())) {
            $this->service->edit($model->id, $model->work_list_id, $model->work_unit_cost, $model->work_scope);
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing ConsumptionRate model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @throws NotFoundHttpException if the model cannot be found
     * @throws StaleObjectException
     * @throws Throwable
     */
    public function actionDelete(int $id): Response
    {
        try {
            $this->findModel($id)->delete();
        } catch (IntegrityException $e) {
            throw new ForbiddenHttpException('Данную запись нельзя удалять');
        }

        return $this->redirect(['index']);
    }

    /**
     * Finds the ConsumptionRate model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @return ConsumptionRate the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel(int $id): ConsumptionRate
    {
        if ($model = ConsumptionRate::findOne($id)) {
            return $model;
        }

        throw new NotFoundHttpException('Запись не найдена.');
    }
}
