<?php

namespace backend\controllers\dictionaries;

use backend\controllers\CrudController;
use common\entities\Dictionaries\WorkListsMaterial;
use common\services\WorkListMaterialService;
use DomainException;
use Throwable;
use Yii;
use common\entities\Dictionaries\WorkList;
use yii\base\Model;
use yii\base\Module;
use yii\bootstrap4\ActiveForm;
use yii\data\ActiveDataProvider;
use yii\db\IntegrityException;
use yii\db\StaleObjectException;
use yii\web\ForbiddenHttpException;
use yii\web\NotFoundHttpException;
use yii\web\Response;

/**
 * WorkListController реализует CRUD операции над перечнем работ.
 */
class WorkListController extends CrudController
{
    private WorkListMaterialService $materialService;

    /** @param array<string> $config */
    public function __construct(string $id, Module $module, WorkListMaterialService $service, array $config = [])
    {
        parent::__construct($id, $module, $config);

        $this->materialService = $service;
    }

    /**
     * Lists all WorkList models.
     */
    public function actionIndex(): string
    {
        $dataProvider = new ActiveDataProvider([
            'query' => WorkList::find(),
        ]);

        return $this->render('index', [
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single WorkList model.
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView(int $id): string
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new WorkList model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return string|Response|array
     */
    public function actionCreate()
    {
        $model = new WorkList();

        $request = Yii::$app->getRequest();
        $data = Yii::$app->request->post('WorkListsMaterial', []);
        if ($request->isPost && $request->post('ajax') !== null) {
            $result = $this->materialValidation($data);
            Yii::$app->response->format = Response::FORMAT_JSON;
            return $result;
        }

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            try {
                $this->materialService->batchUpdate($data, $model);
                return $this->redirect(['view', 'id' => $model->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing WorkList model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @return string|Response|array
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate(int $id, ?int $rate = null)
    {
        $model = $this->findModel($id);

        $request = Yii::$app->getRequest();
        $data = Yii::$app->request->post('WorkListsMaterial', []);
        if ($request->isPost && $request->post('ajax') !== null) {
            $result = $this->materialValidation($data);
            Yii::$app->response->format = Response::FORMAT_JSON;

            return $result;
        }

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            try {
                $this->materialService->batchUpdate($data, $model);
                if ($rate) {
                    return $this->redirect(['consumption-rate/update-materials', 'id' => $rate]);
                }

                return $this->redirect(['view', 'id' => $model->id]);
            } catch (DomainException $e) {
                Yii::$app->errorHandler->logException($e);
                Yii::$app->session->setFlash('error', $e->getMessage());
            }
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Валидация полей материалов
     * @param array<string|int> $data
     * @return array
     */
    public function materialValidation(array $data): array
    {
        $workListMaterials = [new WorkListsMaterial()];
        foreach (array_keys($data) as $index) {
            $workListMaterials[$index] = new WorkListsMaterial();
        }
        Model::loadMultiple($workListMaterials, Yii::$app->request->post());

        return ActiveForm::validateMultiple($workListMaterials);
    }

    /**
     * Deletes an existing WorkList model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @throws NotFoundHttpException if the model cannot be found
     * @throws StaleObjectException
     * @throws Throwable
     */
    public function actionDelete(int $id): Response
    {
        try {
            $this->findModel($id)->delete();
        } catch (IntegrityException $e) {
            throw new ForbiddenHttpException('Данную запись нельзя удалять');
        }

        return $this->redirect(['index']);
    }

    /**
     * Finds the WorkList model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @return WorkList the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel(int $id): WorkList
    {
        if ($model = WorkList::findOne($id)) {
            return $model;
        }

        throw new NotFoundHttpException('Страница не найдена.');
    }
}
