<?php

namespace backend\tests\unit;

use Imagick;
use ImagickException;
use PHPUnit\Framework\TestCase;
use common\entities\Project\PdfInvalidFormat;
use common\entities\Project\PdfPageDoesNotExist;
use common\entities\Project\PdfDoesNotExist;
use common\entities\Project\PdfParser;

class ProjectImageTest extends TestCase
{
    protected string $testFile;
    protected string $multiPageTestFile;

    public function setUp(): void
    {
        parent::setUp();

        $this->testFile = codecept_data_dir() . 'test.pdf';
        $this->multiPageTestFile = codecept_data_dir() . 'multiPage-test.pdf';
    }

    public function testNonExistingFile()
    {
        $this->expectException(PdfDoesNotExist::class);

        new PdfParser('pdfDoesNotExists.pdf');
    }

    /**
     * @throws PdfDoesNotExist
     * @throws ImagickException
     */
    public function testInvalidFileType()
    {
        $this->expectException(PdfInvalidFormat::class);

        (new PdfParser($this->testFile))->setOutputFormat('bla');
    }

    /**
     * @dataProvider invalidPageNumberProvider
     * @param $invalidPage
     * @throws ImagickException
     * @throws PdfDoesNotExist
     * @throws PdfPageDoesNotExist
     */
    public function testInvalidPage($invalidPage)
    {
        $this->expectException(PdfPageDoesNotExist::class);

        (new PdfParser($this->testFile))->setPage($invalidPage);
    }

    /**
     * @throws PdfDoesNotExist
     * @throws ImagickException
     */
    public function testNumberOfPages()
    {
        $pdf = new PdfParser($this->multiPageTestFile);

        $this->assertTrue($pdf->getNumberOfPages() === 3);
    }

    /**
     * @throws ImagickException
     * @throws PdfDoesNotExist
     */
    public function testSpecifiedResolution()
    {
        $image = (new PdfParser($this->testFile))
            ->setResolution(150)
            ->getImageData('test.jpg')
            ->getImageResolution();

        $this->assertEquals(150, $image['x']);
        $this->assertEquals(150, $image['y']);
    }

    /**
     * @throws ImagickException
     * @throws PdfPageDoesNotExist|PdfDoesNotExist
     */
    public function testSpecifiedPage()
    {
        $imagick = (new PdfParser($this->multiPageTestFile))
            ->setPage(2)
            ->getImageData('page-2.jpg');

        $this->assertInstanceOf('Imagick', $imagick);
    }

    /**
     * @throws ImagickException
     * @throws PdfDoesNotExist
     * @throws PdfInvalidFormat
     */
    public function testFileTypeConvert()
    {
        $imagick = (new PdfParser($this->testFile))
            ->setOutputFormat('png')
            ->getImageData('test.png');

        $this->assertSame($imagick->getFormat(), 'png');
        $this->assertNotSame($imagick->getFormat(), 'jpg');
    }

    /**
     * @throws ImagickException
     * @throws PdfDoesNotExist
     */
    public function testAcceptLayer()
    {
        $image = (new PdfParser($this->testFile))
            ->setLayerMethod(Imagick::LAYERMETHOD_FLATTEN)
            ->setResolution(72)
            ->getImageData('test.jpg')
            ->getImageResolution();

        $this->assertEquals(72, $image['x']);
        $this->assertEquals(72, $image['y']);
    }

    /**
     * @throws PdfDoesNotExist
     * @throws ImagickException
     */
    public function testCompressionQuality()
    {
        $imagick = (new PdfParser($this->testFile))
            ->setCompressionQuality(99)
            ->getImageData('test.jpg');

        $this->assertEquals(99, $imagick->getCompressionQuality());
    }

    public function invalidPageNumberProvider(): array
    {
        return [[5], [0], [-1]];
    }
}
