<?php

namespace common\entities\Company;

use common\entities\behaviors\JsonBehavior;
use common\entities\Contract\Contract;
use common\entities\Contract\ContractWork;
use common\forms\Company\CompanyForm;
use yii\behaviors\TimestampBehavior;
use yii\db\ActiveQuery;
use yii\db\ActiveRecord;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "companies".
 *
 * @property int $id
 * @property string $full_name Полное наименование
 * @property string $short_name Короткое наименование
 * @property string $delegate_name ФИО представителя
 * @property string|null $delegate_phones Контактные телефоны представителя
 * @property string $delegate_email Почта представителя
 * @property string $signer_post Должность подписанта
 * @property string $signer_name ФИО подписанта
 * @property string|null $general_phones Общие телефоны компании
 * @property string|null $general_email Общая почта
 * @property string $legal_adress Юридический адрес
 * @property string $physical_adress Фактический адрес
 * @property int $company_type Тип организации
 * @property string $inn ИНН
 * @property string $ogrn ОГРН
 * @property string $okpo ОКПО
 * @property string $kpp КПП
 * @property string|null $company_url Ссылка на сайт
 * @property string $payment_account Р/С
 * @property string $bank Банк
 * @property string $bank_cor_account К/С
 * @property string $bank_bic БИК
 * @property bool $is_client Является клиентом
 * @property int $created_at Дата созданипя
 * @property int $updated_at Дата обновления
 *
 * @property Contract[] $contracts
 */
class Company extends ActiveRecord
{
    public static function tableName(): string
    {
        return '{{%companies}}';
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels(): array
    {
        return CompanyForm::instance()->attributeLabels();
    }

    /**
     * {@inheritdoc}
     */
    public function rules(): array
    {
        return CompanyForm::instance()->rules();
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors(): array
    {
        return [
            TimestampBehavior::class,
            [
                'class' => JsonBehavior::class,
                'attributes' => ['delegate_phones', 'general_phones'],
            ],
        ];
    }

    /**
     * Создание/Редактирование записи.
     */
    public function create($form): void
    {
        foreach ($form as $field => $value) {
            $this->$field = $value;
        }
    }

    /**
     * Проверка, является ли юридическое лицо клиентом.
     */
    public function isClient(): bool
    {
        return $this->is_client === true;
    }

    /**
     * Массив юридических лиц по ID и полному наименованию.
     */
    public static function companiesList(): array
    {
        return ArrayHelper::map(self::find()->all(), 'id', 'full_name');
    }

    /**
     * Gets query for [[ContractCompanies]].
     */
    public function getContracts(): ?ActiveQuery
    {
        return $this->hasMany(Contract::class, ['company_id' => 'id']);
    }

    /**
     * Получить договоры со статусом "Действующий".
     */
    public function getContractsActivity(): ?ActiveQuery
    {
        return Contract::find()->where(['company_id' => $this->id])->andWhere(['status' => Contract::STATUS_ACTIVE]);
    }

    /**
     * Получить наибольшую дату окончания работы по договорам.
     */
    public function getContractsMaxDateEnd(): ?string
    {
        $ids = [];
        foreach ($this->contracts as $contract) {
            $ids[] = $contract->id;
        }

        return ContractWork::find()->where(['contract_id' => $ids])->max('date_end');
    }
}
