<?php

namespace common\entities\Contract;

use common\entities\Company\Company;
use common\forms\Contract\ContractAmountForm;
use common\forms\Contract\ContractForm;
use common\helpers\DateHelper;
use common\helpers\DayHelper;
use common\modules\entitiesHistory\behaviors\HistoryBehavior;
use Exception;
use Yii;
use yii\base\InvalidConfigException;
use yii\behaviors\TimestampBehavior;
use yii\db\ActiveQuery;
use yii\db\ActiveRecord;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "{{%contracts}}".
 *
 * @property int $id
 * @property int $company_id Запись юридического лица
 * @property string $number Номер договора
 * @property string $date_start Дата договора
 * @property string|null $subject_address Адрес объекта
 * @property string|null $subject_name Наименование объекта
 * @property bool $is_act_counter_increase_annually Обновление счетчика акта после нг
 * @property int $documents_day_end Число закрытия документов
 * @property int $status Статус
 * @property int|null $is_prepayment Авансовый платеж
 * @property int|null $prepayment_percent Процент авансового платежа
 * @property int|null $report_start_day Число начала отчетного периода
 * @property int|null $report_end_day Число окончания отчетного периода
 * @property string|null $date_works_start Дата начала работ
 * @property string|null $date_works_end Дата окончания работ
 * @property float|null $amount Общая стоимость
 * @property bool $is_nds НДС
 * @property int $created_at Дата создания
 * @property int $updated_at Дата обновления
 *
 * @property ContractCompany[] $contractCompanies
 * @property Company $company
 * @property ContractExpense[] $contractExpenses
 * @property ContractWork[] $contractWorks
 */
class Contract extends ActiveRecord
{
    public const STATUS_WAIT = 10;
    public const STATUS_ACTIVE = 20;
    public const STATUS_UNCOMPLETED = 30;
    public const STATUS_FINISH = 40;
    public const STATUS_ARCHIVED = 50;

    /**
     * Создание записи.
     */
    public static function create(ContractForm $form): self
    {
        $contract = new static();
        foreach ($form as $field => $value) {
            $contract->$field = $value;
        }
        if (empty($contract->status)) {
            $contract->status = 30;
        }

        return $contract;
    }

    public static function tableName(): string
    {
        return '{{%contracts}}';
    }

    /** @throws InvalidConfigException */
    public function beforeSave($insert): bool
    {
        if (parent::beforeSave($insert)) {
            $this->date_start = DateHelper::convert($this->date_start);
            $this->date_works_start = DateHelper::convert($this->date_works_start);
            $this->date_works_end = DateHelper::convert($this->date_works_end);

            return true;
        }

        return false;
    }

    /**
     * Редактирование записи.
     */
    public function edit($form): void
    {
        foreach ($form as $field => $value) {
            $this->$field = $value;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors(): array
    {
        return [
            TimestampBehavior::class,
            'History' => [
                'class' => HistoryBehavior::class,
                'allowEvents' => [
                    //HistoryBehavior::EVENT_INSERT,
                    HistoryBehavior::EVENT_UPDATE,
                    //HistoryBehavior::EVENT_DELETE,
                ],
                'skipAttributes' => [
                    'number',
                    'date_start',
                    'subject_address',
                    'subject_name',
                    'is_act_counter_increase_annually',
                    'documents_day_end',
                    'status',
                    'created_at',
                    'updated_at',
                ],
            ],
        ];
    }

    /**
     * Возвращает массив статусов договора.
     */
    public static function statusList(): array
    {
        return [
            Contract::STATUS_WAIT => 'Ожидается старт работ',
            Contract::STATUS_ACTIVE => 'Действующий',
            Contract::STATUS_UNCOMPLETED => 'Черновик',
            Contract::STATUS_FINISH => 'Завершен',
            Contract::STATUS_ARCHIVED => 'Архивный',
        ];
    }

    /**
     * Получение значения статуса.
     *
     * @throws Exception
     */
    public function getStatusName(): string
    {
        return ArrayHelper::getValue(self::statusList(), $this->status);
    }

    /**
     * @throws InvalidConfigException
     * @throws Exception
     */
    public static function historyFormatter($dataProvider, $value): ?string
    {
        if ($dataProvider->field_name === 'is_nds') {
            return $dataProvider->$value === '1' ? 'Да' : 'Нет';
        } elseif ($dataProvider->field_name === 'date_works_start' || $dataProvider->field_name === 'date_works_end') {
            return Yii::$app->formatter->asDate($dataProvider->$value, 'php:d.m.Y');
        } elseif ($dataProvider->field_name === 'report_start_day' || $dataProvider->field_name === 'report_end_day') {
            return DayHelper::isLastDay($dataProvider->$value);
        } elseif ($dataProvider->field_name === 'is_prepayment') {
            return $dataProvider->$value === '1' ? 'Да' : 'Нет';
        }

        return $dataProvider->$value;
    }

    /**
     * Gets query for [[ContractCompanies]].
     */
    public function getContractCompanies(): ?ActiveQuery
    {
        return $this->hasMany(ContractCompany::class, ['contract_id' => 'id']);
    }

    /**
     * Gets query for [[Company]].
     */
    public function getCompany(): ?ActiveQuery
    {
        return $this->hasOne(Company::class, ['id' => 'company_id']);
    }

    /**
     * Gets query for [[ContractExpenses]].
     */
    public function getContractExpenses(): ?ActiveQuery
    {
        return $this->hasMany(ContractExpense::class, ['contract_id' => 'id']);
    }

    /**
     * Gets query for [[ContractWorks]].
     */
    public function getContractWorks(): ?ActiveQuery
    {
        return $this->hasMany(ContractWork::class, ['contract_id' => 'id']);
    }

    /**
     * Получение наибольшей даты завершения работ по договору.
     */
    public function getLastDateEndContractWorks(): ?string
    {
        return $this->hasMany(ContractWork::class, ['contract_id' => 'id'])->max('date_end');
    }

    /**
     * Получение прогресса по договору.
     */
    public function getProgress(): int
    {
        $today = time();
        $dateOne = strtotime($this->date_start);
        $dateTwo = strtotime($this->lastDateEndContractWorks);
        if ($today < $dateOne) {
            $percentage = 0;
        } elseif ($today > $dateTwo) {
            $percentage = 100;
        } else {
            $percentage = ($today - $dateOne) * 100 / ($dateTwo - $dateOne);
        }

        return $percentage;
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels(): array
    {
        return ArrayHelper::merge(ContractForm::instance()->attributeLabels(), ContractAmountForm::instance()->attributeLabels());
    }

    /**
     * {@inheritdoc}
     */
    public function rules(): array
    {
        return ContractForm::instance()->rules();
    }
}
