<?php

namespace common\entities\Contract;

use common\entities\Company\Company;
use common\entities\Dictionaries\WorkList;
use common\entities\Dictionaries\WorkType;
use common\forms\Contract\ContractEstimateForm;
use common\helpers\DateHelper;
use common\modules\entitiesHistory\behaviors\HistoryBehavior;
use Exception;
use Yii;
use yii\base\InvalidConfigException;
use yii\behaviors\TimestampBehavior;
use yii\db\ActiveQuery;
use yii\db\ActiveRecord;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "{{%contract_estimates}}".
 *
 * @property int $id
 * @property int $contract_id Запись договора
 * @property int $work_type_id Запись типа работы
 * @property int $surface Поверхность
 * @property string $bill_number Номер счета/накладной
 * @property string|null $bill_date Дата счета/накладной
 * @property int $work_list_id Запись работы
 * @property int $type Тип работы
 * @property int $contractor_id Ответственный за исполнение
 * @property float $quantity Количество
 * @property float $price_per_unit Цена за единицу
 * @property int|null $markup Наценка
 * @property int $created_at Дата создания
 * @property int $updated_at Дата обновления
 *
 * @property Contract $contract
 * @property WorkType $workType
 * @property WorkList $workList
 * @property ContractCompany $contractor
 */
class ContractEstimate extends ActiveRecord
{
    public const TYPE_MATERIALS = 10;
    public const TYPE_EXP_MATERIALS = 20;
    public const TYPE_EQUIPMENT = 30;
    public const TYPE_TRANSPORT = 40;
    public const TYPE_SALARY = 50;

    public const SURFACE_WALL = 10;
    public const SURFACE_FLOOR = 20;
    public const SURFACE_CEIL = 30;

    /**
     * Создание записи.
     */
    public static function create(ContractEstimateForm $form): self
    {
        $estimate = new static();
        foreach ($form as $field => $value) {
            $estimate->$field = $value;
        }

        return $estimate;
    }

    public static function tableName(): string
    {
        return '{{%contract_estimates}}';
    }

    /** @throws InvalidConfigException */
    public function beforeSave($insert): bool
    {
        if (parent::beforeSave($insert)) {
            $this->bill_date = DateHelper::convert($this->bill_date);

            return true;
        }

        return false;
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors(): array
    {
        return [
            TimestampBehavior::class,
            'History' => [
                'class' => HistoryBehavior::class,
                'allowEvents' => [
                    //HistoryBehavior::EVENT_INSERT,
                    HistoryBehavior::EVENT_UPDATE,
                    //HistoryBehavior::EVENT_DELETE,
                ],
                'skipAttributes' => [
                    'created_at',
                    'updated_at',
                ],
            ],
        ];
    }

    /**
     * Редактирование записи.
     */
    public function edit($form): void
    {
        foreach ($form as $field => $value) {
            $this->$field = $value;
        }
    }

    /**
     * Возвращает массив типов расходов по этапу сметы.
     */
    public static function typeList(): array
    {
        return [
            self::TYPE_MATERIALS => 'Материалы',
            self::TYPE_EXP_MATERIALS => 'Расходные материалы',
            self::TYPE_EQUIPMENT => 'Оборудование',
            self::TYPE_TRANSPORT => 'Транспорт',
            self::TYPE_SALARY => 'Оплата труда',
        ];
    }

    /**
     * Получение значения типа расхода по этапу смметы.
     *
     * @throws Exception
     */
    public function getTypeName(): string
    {
        return ArrayHelper::getValue(self::typeList(), $this->type);
    }

    /**
     * Возвращает массив поверхностей по этапу сметы.
     */
    public static function surfaceList(): array
    {
        return [
            self::SURFACE_WALL => 'Стены',
            self::SURFACE_FLOOR => 'Пол',
            self::SURFACE_CEIL => 'Потолок',
        ];
    }

    /**
     * Получение значения поверхности.
     *
     * @throws Exception
     */
    public function getSurfaceName(): string
    {
        return ArrayHelper::getValue(self::surfaceList(), $this->surface);
    }

    /**
     * Общая сумма по этапу сметы.
     */
    public function estimateAmount(): float
    {
        $amount = $this->quantity * $this->price_per_unit;
        $percent = $amount * $this->markup / 100;

        return $amount + $percent;
    }

    /**
     * Общая сумма по этапу сметы с НДС.
     */
    public function estimateAmountWithNds(): float
    {
        $amount = $this->quantity * $this->price_per_unit;
        $percent = $amount * $this->markup / 100;
        $nds = ($amount + $percent) * 20 / 100;

        return $amount + $percent + $nds;
    }

    /**
     * @throws InvalidConfigException
     * @throws Exception
     */
    public static function historyFormatter($dataProvider, $value): string
    {
        if ($dataProvider->field_name === 'bill_date') {
            return Yii::$app->formatter->asDate($dataProvider->$value);
        } elseif ($dataProvider->field_name === 'surface') {
            return ContractEstimateForm::surfaceName($dataProvider->$value);
        } elseif ($dataProvider->field_name === 'work_list_id') {
            if ($model = WorkList::findOne($dataProvider->$value)) {
                return $model->name;
            }
            return 'Отсутствует';
        } elseif ($dataProvider->field_name === 'type') {
            return ContractEstimateForm::typeName($dataProvider->$value);
        } elseif ($dataProvider->field_name === 'contractor_id') {
            if ($model = Company::findOne($dataProvider->$value)) {
                return $model->full_name;
            }
            return 'Отсутствует';
        }

        return $dataProvider->$value;
    }

    /**
     * Gets query for [[Contract]].
     */
    public function getContract(): ActiveQuery
    {
        return $this->hasOne(Contract::class, ['id' => 'contract_id']);
    }

    /**
     * Gets query for [[WorkType]].
     */
    public function getWorkType(): ActiveQuery
    {
        return $this->hasOne(WorkType::class, ['id' => 'work_type_id']);
    }

    /**
     * Gets query for [[WorkList]].
     */
    public function getWorkList(): ActiveQuery
    {
        return $this->hasOne(WorkList::class, ['id' => 'work_list_id']);
    }

    /**
     * Gets query for [[Contractor]].
     */
    public function getContractor(): ActiveQuery
    {
        return $this->hasOne(ContractCompany::class, ['company_id' => 'contractor_id']);
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels(): array
    {
        return ContractEstimateForm::instance()->attributeLabels();
    }

    /**
     * {@inheritdoc}
     */
    public function rules(): array
    {
        return ContractEstimateForm::instance()->rules();
    }
}
