<?php

namespace common\entities\Contract;

use common\entities\Company\Company;
use common\helpers\ContractExpenseHelper;
use common\modules\entitiesHistory\behaviors\HistoryBehavior;
use common\modules\entitiesHistory\entities\History;
use Exception;
use ReflectionClass;
use yii\base\InvalidConfigException;
use yii\db\ActiveQuery;
use yii\db\ActiveRecord;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "{{%contract_expenses}}".
 *
 * @property int $id
 * @property int $contract_id Запись договора
 * @property int $expense_type Тип расхода
 * @property float $amount Сумма
 * @property bool $is_nds НДС
 * @property int $contractor_id Ответственный за покупку
 * @property string $comment Дополнительный комментарий
 *
 * @property Contract $contract
 * @property ContractCompany $contractor
 */
class ContractExpense extends ActiveRecord
{
    public const TYPE_MATERIALS = 10;
    public const TYPE_EXP_MATERIALS = 20;
    public const TYPE_EQUIPMENT = 30;
    public const TYPE_TRANSPORT = 40;

    public static function tableName(): string
    {
        return '{{%contract_expenses}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules(): array
    {
        return [
            [['contract_id', 'expense_type', 'amount', 'contractor_id', 'comment'], 'required'],
            [['contract_id', 'contractor_id', 'expense_type', 'is_nds'], 'integer'],
            [['amount'], 'number', 'min' => 0],
            [['comment'], 'string', 'max' => 512],
            [['contract_id'], 'exist', 'skipOnError' => true, 'targetClass' => Contract::class, 'targetAttribute' => ['contract_id' => 'id']],
            [
                [
                    'contractor_id',
                ],
                'exist', 'skipOnError' => true, 'targetClass' => ContractCompany::class, 'targetAttribute' => ['contractor_id' => 'company_id'],
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels(): array
    {
        return [
            'id' => 'ID',
            'contract_id' => 'Запись договора',
            'expense_type' => 'Тип расхода',
            'amount' => 'Сумма',
            'is_nds' => 'НДС',
            'contractor_id' => 'Ответственный за покупку',
            'comment' => 'Дополнительный комментарий',
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors(): array
    {
        return [
            'History' => [
                'class' => HistoryBehavior::class,
                'allowEvents' => [
                    //HistoryBehavior::EVENT_INSERT,
                    HistoryBehavior::EVENT_UPDATE,
                    //HistoryBehavior::EVENT_DELETE,
                ],
            ],
        ];
    }

    /**
     * Возвращает массив типов расходов по договору.
     */
    public static function typeList(): array
    {
        return [
            self::TYPE_MATERIALS => 'Материалы',
            self::TYPE_EXP_MATERIALS => 'Расходные материалы',
            self::TYPE_EQUIPMENT => 'Оборудование',
            self::TYPE_TRANSPORT => 'Транспорт',
        ];
    }

    /**
     * Получение значения типа расхода по договору.
     *
     * @throws Exception
     */
    public function getTypeName(): string
    {
        return ArrayHelper::getValue(self::typeList(), $this->expense_type);
    }

    /**
     * @throws InvalidConfigException
     * @throws Exception
     */
    public static function historyFormatter($dataProvider, $value): string
    {
        if ($dataProvider->field_name === 'expense_type') {
            return ContractExpenseHelper::typeName($dataProvider->$value);
        } elseif ($dataProvider->field_name === 'is_nds') {
            return $dataProvider->$value === '1' ? 'Да' : 'Нет';
        } elseif ($dataProvider->field_name === 'contractor_id') {
            if ($model = Company::findOne($dataProvider->$value)) {
                return $model->full_name;
            }

            return 'Отсутствует';
        }

        return $dataProvider->$value;
    }

    public function addNds(): int
    {
        return $this->amount + ($this->amount * 0.2);
    }

    public function removeNds(): int
    {
        return $this->amount - ($this->amount * 20 / 120);
    }

    /**
     * Проверка на признак редактирования связанного расхода.
     */
    public function getHistory(): bool
    {
        return History::find()->where(['model_name' => (new ReflectionClass($this))->name, 'row_id' => $this->id])->exists();
    }

    /**
     * Gets query for [[Contract]].
     */
    public function getContract(): ActiveQuery
    {
        return $this->hasOne(Contract::class, ['id' => 'contract_id']);
    }

    /**
     * Gets query for [[Contractor]].
     */
    public function getContractor(): ActiveQuery
    {
        return $this->hasOne(ContractCompany::class, ['company_id' => 'contractor_id']);
    }
}
