<?php

namespace common\entities\Project;

use Imagick;
use ImagickException;

class PdfParser
{
    protected string $pdfFile;

    protected int $resolution = 144;

    protected string $outputFormat = 'jpg';

    protected int $page = 1;

    public Imagick $imagick;

    protected int $numberOfPages;

    /** @var array<string> */
    protected array $validOutputFormats = ['jpg', 'jpeg', 'png'];

    protected ?int $layerMethod = Imagick::LAYERMETHOD_FLATTEN;

    /** @var string|int|null $colorspace */
    protected $colorspace = null;

    /** @var string|int|null $compressionQuality */
    protected $compressionQuality = null;

    /**
     * @throws ImagickException
     * @throws PdfDoesNotExist
     */
    public function __construct(string $pdfFile, ?int $pageCount = null)
    {
        if (!file_exists($pdfFile)) {
            throw new PdfDoesNotExist("Файл `{$pdfFile}` не существует");
        }

        if (!$pageCount) {
            $this->imagick = new Imagick();
            $this->imagick->pingImage($pdfFile);
            $this->numberOfPages = $this->imagick->getNumberImages();
        } else {
            $this->numberOfPages = $pageCount;
        }


        $this->pdfFile = $pdfFile;
    }

    public function setResolution(int $resolution): PdfParser
    {
        $this->resolution = $resolution;

        return $this;
    }

    /** @throws PdfInvalidFormat */
    public function setOutputFormat(string $outputFormat): PdfParser
    {
        if (!$this->isValidOutputFormat($outputFormat)) {
            throw new PdfInvalidFormat("Формат {$outputFormat} не поддерживается");
        }

        $this->outputFormat = $outputFormat;

        return $this;
    }

    public function getOutputFormat(): string
    {
        return $this->outputFormat;
    }

    /**
     * Sets the layer method for Imagick::mergeImageLayers()
     * If int, should correspond to a predefined LAYERMETHOD constant.
     * If null, Imagick::mergeImageLayers() will not be called.
     *
     * @param int|null
     *
     * @return $this
     *
     * @see https://secure.php.net/manual/en/imagick.constants.php
     * @see Pdf::getImageData()
     */
    public function setLayerMethod(?int $layerMethod): PdfParser
    {
        $this->layerMethod = $layerMethod;

        return $this;
    }

    public function isValidOutputFormat(string $outputFormat): bool
    {
        return in_array($outputFormat, $this->validOutputFormats);
    }

    /** @throws PdfPageDoesNotExist */
    public function setPage(int $page): PdfParser
    {
        if ($page > $this->getNumberOfPages() || $page < 1) {
            throw new PdfPageDoesNotExist("Страница {$page} отсутствует");
        }

        $this->page = $page;

        return $this;
    }

    public function getNumberOfPages(): int
    {
        return $this->numberOfPages;
    }

    /**
     * @throws ImagickException
     * @return bool|string
     */
    public function saveImage(string $pathToImage)
    {
        if (is_dir($pathToImage)) {
            $pathToImage = rtrim($pathToImage, '\/') . DIRECTORY_SEPARATOR . $this->page . '.' . $this->outputFormat;
        }

        $imageData = $this->getImageData($pathToImage);

        if (file_put_contents($pathToImage, $imageData) !== false) {
            return $pathToImage;
        }

        return false;
    }

    public function saveAllPagesAsImages(string $directory, string $prefix = ''): array
    {
        $numberOfPages = $this->getNumberOfPages();

        if ($numberOfPages === 0) {
            return [];
        }

        return array_map(function ($pageNumber) use ($directory, $prefix) {
            $this->setPage($pageNumber);

            $destination = "{$directory}/{$prefix}{$pageNumber}.{$this->outputFormat}";

            $this->saveImage($destination);

            return $destination;
        }, range(1, $numberOfPages));
    }

    /** @throws ImagickException */
    public function getImageData(string $pathToImage): Imagick
    {
        /*
         * Reinitialize imagick because the target resolution must be set
         * before reading the actual image.
         */
        $this->imagick = new Imagick();

        $this->imagick->setResolution($this->resolution, $this->resolution);

        if ($this->colorspace !== null) {
            $this->imagick->setColorspace($this->colorspace);
        }

        if ($this->compressionQuality !== null) {
            $this->imagick->setCompressionQuality($this->compressionQuality);
        }

        if (filter_var($this->pdfFile, FILTER_VALIDATE_URL)) {
            return $this->getRemoteImageData($pathToImage);
        }

        $this->imagick->readImage(sprintf('%s[%s]', $this->pdfFile, $this->page - 1));

        if (is_int($this->layerMethod)) {
            $this->imagick = $this->imagick->mergeImageLayers($this->layerMethod);
        }

        $this->imagick->setFormat($this->determineOutputFormat($pathToImage));

        return $this->imagick;
    }

    public function setColorspace(int $colorspace): PdfParser
    {
        $this->colorspace = $colorspace;

        return $this;
    }

    public function setCompressionQuality(int $compressionQuality): PdfParser
    {
        $this->compressionQuality = $compressionQuality;

        return $this;
    }

    /** @throws ImagickException */
    protected function getRemoteImageData(string $pathToImage): Imagick
    {
        $this->imagick->readImage($this->pdfFile);

        $this->imagick->setIteratorIndex($this->page - 1);

        if (is_int($this->layerMethod)) {
            $this->imagick = $this->imagick->mergeImageLayers($this->layerMethod);
        }

        $this->imagick->setFormat($this->determineOutputFormat($pathToImage));

        return $this->imagick;
    }

    protected function determineOutputFormat(string $pathToImage): string
    {
        $outputFormat = pathinfo($pathToImage, PATHINFO_EXTENSION);

        if ($this->outputFormat != '') {
            $outputFormat = $this->outputFormat;
        }

        $outputFormat = strtolower($outputFormat);

        if (!$this->isValidOutputFormat($outputFormat)) {
            $outputFormat = 'jpg';
        }

        return $outputFormat;
    }
}
