<?php

namespace common\entities\Project;

use common\entities\Contract\Contract;
use common\entities\Dictionaries\WorkType;
use common\helpers\ProjectImageHelper;
use yii\db\ActiveQuery;
use yii\db\ActiveRecord;
use yii\helpers\ArrayHelper;
use yii\web\HttpException;

/**
 * This is the model class for table "{{%project_images}}".
 *
 * @property int $id
 * @property string $name Наименование изображения
 * @property int $contract_id Запись договора
 * @property int $work_type_id Запись типа работы
 * @property string $pdf_name Наименование pdf файла
 * @property int $is_info_page Информационная страница
 * @property int $stage Этаж
 *
 * @property Contract $contract
 * @property WorkType $workType
 */
class ProjectImage extends ActiveRecord
{
    /**
     * Создание записей.
     */
    public static function create(string $page, int $contractId, string $pdfName): self
    {
        $image = self::find()->where(['contract_id' => $contractId, 'pdf_name' => $pdfName, 'name' => basename($page)])->limit(1)->one();

        if (!$image) {
            $image = new self();
            $image->contract_id = $contractId;
            $image->name = basename($page);
            $image->pdf_name = $pdfName;
            $image->save();
        }

        return $image;
    }

    public static function tableName(): string
    {
        return '{{%project_images}}';
    }

    /**
     * {@inheritdoc}
     */
    public function rules(): array
    {
        return [
            [['name', 'contract_id', 'pdf_name'], 'required'],
            [['contract_id', 'work_type_id', 'is_info_page'], 'integer'],
            [['name', 'pdf_name'], 'unique', 'targetAttribute' => ['name', 'pdf_name', 'contract_id']],
            [['name', 'pdf_name'], 'string', 'max' => 255],
            [['stage'], 'integer', 'min' => 0, 'max' => 999],
            [['contract_id'], 'exist', 'skipOnError' => true, 'targetClass' => Contract::class, 'targetAttribute' => ['contract_id' => 'id']],
            [['work_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => WorkType::class, 'targetAttribute' => ['work_type_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels(): array
    {
        return [
            'id' => 'ID',
            'name' => 'Наименование изображения',
            'contract_id' => 'Запись договора',
            'work_type_id' => 'Запись типа работы',
            'pdf_name' => 'Наименование pdf файла',
            'stage' => 'Этаж',
        ];
    }

    public function afterFind()
    {
        if ($this->is_info_page === 1) {
            $this->work_type_id = ProjectImageHelper::TYPE_INFO;
        }

        parent::afterFind();
    }

    /** @throws HttpException */
    public static function addStage(int $id, int $stage): void
    {
        $image = self::find()->where(['id' => $id])->limit(1)->one();

        if ($image) {
            $image->stage = $stage;
        } else {
            throw new HttpException(404, 'Изображение не найдено');
        }
    }

    /**
     * Редактирование записей.
     */
    public static function edit($images): array
    {
        $arr = [];
        foreach ($images as $image) {
            $model = ProjectImage::findOne(['id' => $image['id']]);
            if ($image['work_type_id'] == ProjectImageHelper::TYPE_INFO) {
                $model->work_type_id = null;
                $model->is_info_page = 1;
            } else {
                $model->work_type_id = $image['work_type_id'];
                $model->is_info_page = 0;
            }
            $model->stage = $image['stage'];

            $model->save();
            $arr[] = $model;
        }

        return $arr;
    }

    /**
     * @param $contractId
     * @return ProjectImage|array|null
     */
    public function getImages($contractId): ?array
    {
        return self::findAll(['contract_id' => (int) $contractId]);
    }

    /**
     * Массив изображений по ID и наименованию.
     */
    public function imagesList($contractId): array
    {
        return ArrayHelper::map(self::getImages($contractId), 'name', 'name');
    }

    /**
     * Gets query for [[Contract]].
     */
    public function getContract(): ActiveQuery
    {
        return $this->hasOne(Contract::class, ['id' => 'contract_id']);
    }

    /**
     * Gets query for [[WorkType]].
     */
    public function getWorkType(): ActiveQuery
    {
        return $this->hasOne(WorkType::class, ['id' => 'work_type_id']);
    }
}
