<?php

namespace common\helpers;

use Exception;
use Imagick;
use ImagickDraw;
use ImagickDrawException;
use ImagickException;
use ImagickPixel;
use ImagickPixelException;

class ImageHelper
{
    private Imagick $image;
    private int $height;
    private int $width;

    /**
     * @param string $imagePath Путь до изображения.
     * @throws ImagickException
     */
    public static function open(string $imagePath): self
    {
        $model = new self();
        $model->image = new Imagick($imagePath);
        $geo = $model->image->getImageGeometry();
        $model->height = $geo['height'];
        $model->width = $geo['width'];

        return $model;
    }

    /**
     * Получить ширину изображения.
     */
    public function getWidth(): int
    {
        return $this->width;
    }

    /**
     * Получить высоту изображения
     */
    public function getHeight(): int
    {
        return $this->height;
    }

    /**
     * @return void
     * @throws ImagickException
     */
    public function saveTo(string $path)
    {
        $this->image->writeImage($path);
        $this->image->destroy();
    }

    /**
     * Добавить границы.
     *
     * @throws ImagickDrawException
     * @throws ImagickException
     * @throws ImagickPixelException
     */
    public function border(int $width, string $color): ImageHelper
    {
        $border = new ImagickDraw();
        $border->setFillColor(new ImagickPixel($color));
        $border->setStrokeColor(new ImagickPixel($color));
        $border->setStrokeWidth($width);
        $widthPart = $width / 2;
        $border->line(0, 0 + $widthPart, $this->width, 0 + $widthPart);
        $border->line(0, $this->height - $widthPart, $this->width, $this->height - $widthPart);
        $border->line(0 + $widthPart, 0, 0 + $widthPart, $this->height);
        $border->line($this->width - $widthPart, 0, $this->width - $widthPart, $this->height);
        $this->image->drawImage($border);

        return $this;
    }

    /** @throws ImagickException */
    public function blur(float $radius, float $delta): ImageHelper
    {
        $this->image->blurImage($radius, $delta);

        return $this;
    }

    /**
     * Обрезать изображение.
     * @throws ImagickException
     */
    public function crop(int $startX, int $startY, int $width, int $height): ImageHelper
    {
        $this->image->cropImage($width, $height, $startX, $startY);

        return $this;
    }

    /**
     * Вертикальное отзеркаливание.
     * @throws ImagickException
     */
    public function flip(): ImageHelper
    {
        $this->image->flipImage();

        return $this;
    }

    /**
     * Горизонтальное отзеркаливание.
     * @throws ImagickException
     */
    public function flop(): ImageHelper
    {
        $this->image->flopImage();

        return $this;
    }

    /**
     * Добавить водяной знак.
     * @param string $watermarkPath Путь до водяного знака
     * @param string $xPos Горизонтальная позиция - 'left', 'right' или 'center'
     * @param string $yPos Вертикальная позиция - 'top', 'bottom' или 'center'
     * @param bool|int|string $xSize Горизонтальный размер: 100, '50%', 'auto'.
     * @param bool|int|string $ySize Вертикальный размер: 100, '50%', 'auto'.
     * @param bool $xOffset
     * @param bool $yOffset
     * @throws Exception
     */
    public function watermark(
        string $watermarkPath,
        string $xPos,
        string $yPos,
        $xSize = false,
        $ySize = false,
        $xOffset = false,
        $yOffset = false
    ): ImageHelper {
        if ($watermarkPath !== null) {
            $watermark = new Imagick($watermarkPath);

            // изменить размер водяного знака
            $newSizeX = false;
            $newSizeY = false;
            if ($xSize !== false) {
                if (is_numeric($xSize)) {
                    $newSizeX = $xSize;
                } elseif (is_string($xSize) && substr($xSize, -1) === '%') {
                    $float = str_replace('%', '', $xSize) / 100;
                    $newSizeX = $this->width * ((float) $float);
                }
            }
            if ($ySize !== false) {
                if (is_numeric($ySize)) {
                    $newSizeY = $ySize;
                } elseif (is_string($ySize) && substr($ySize, -1) === '%') {
                    $float = str_replace('%', '', $ySize) / 100;
                    $newSizeY = $this->height * ((float) $float);
                }
            }
            if ($newSizeX !== false && $newSizeY !== false) {
                $watermark->adaptiveResizeImage($newSizeX, $newSizeY);
            } elseif ($newSizeX !== false && $newSizeY === false) {
                $watermark->adaptiveResizeImage($newSizeX, 0);
            } elseif ($newSizeX === false && $newSizeY !== false) {
                $watermark->adaptiveResizeImage(0, $newSizeY);
            }

            $startX = false;
            $startY = false;
            $watermarkSize = $watermark->getImageGeometry();
            if ($yPos === 'top') {
                $startY = 0;
                if ($yOffset !== false) {
                    $startY += $yOffset;
                }
            } elseif ($yPos === 'bottom') {
                $startY = $this->height - $watermarkSize['height'];
                if ($yOffset !== false) {
                    $startY -= $yOffset;
                }
            } elseif ($yPos === 'center') {
                $startY = ($this->height / 2) - ($watermarkSize['height'] / 2);
            } else {
                throw new Exception('Параметр $yPos должен быть "top", "bottom" или "center"');
            }

            if ($xPos === 'left') {
                $startX = 0;
                if ($xOffset !== false) {
                    $startX += $xOffset;
                }
            } elseif ($xPos === 'right') {
                $startX = $this->width - $watermarkSize['width'];
                if ($xOffset !== false) {
                    $startX -= $xOffset;
                }
            } elseif ($xPos === 'center') {
                $startX = ($this->width / 2) - ($watermarkSize['width'] / 2);
            } else {
                throw new Exception('Параметр $xPos должен быть "left", "right" или "center"');
            }

            $this->image->compositeImage($watermark, Imagick::COMPOSITE_OVER, $startX, $startY);
        }

        return $this;
    }

    /**
     * Создать эскиз.
     * @throws ImagickException
     */
    public function thumb(int $width, int $height): ImageHelper
    {
        if ($this->width > $width || $this->height > $height) {
            if ($this->width >= $this->height || $height === false) {
                $this->image->thumbnailImage($width, 0);
            } else {
                $this->image->thumbnailImage(0, $height);
            }
        }

        return $this;
    }

    /**
     * Изменить размер.
     * @throws ImagickException
     * @throws Exception
     */
    public function resize(int $width, int $height): ImageHelper
    {
        if ($height == false && $width == false) {
            throw new Exception('$width и $height не могут быть пустыми');
        }
        if ($width != false && $height != false) {
            if ($this->width >= $this->height) {
                $this->image->adaptiveResizeImage($width, 0);
            } else {
                $this->image->adaptiveResizeImage(0, $height);
            }
        } else {
            if ($width == false) {
                $this->image->adaptiveResizeImage(0, $height);
            } elseif ($height == false) {
                $this->image->adaptiveResizeImage($width, 0);
            }
        }

        return $this;
    }
}
