<?php

namespace common\modules\entitiesHistory\behaviors;

use ReflectionClass;
use yii\base\Behavior;
use yii\db\BaseActiveRecord;
use yii\base\Event;
use common\modules\entitiesHistory\HistoryModule;
use common\modules\entitiesHistory\forms\HistoryForm as HistoryEntity;
use yii\base\NotSupportedException;

/**
 * Behavior отслеживает изменения и сохраняет в хранилище.
 */
class HistoryBehavior extends Behavior
{
    public const EVENT_INSERT = 1;
    public const EVENT_UPDATE = 2;
    public const EVENT_DELETE = 3;
    
    /**
     * Допустимые действия для триггера.
     * @var array<int>
     */
    public array $allowEvents = [
        self::EVENT_INSERT,
        self::EVENT_UPDATE,
        self::EVENT_DELETE,
    ];

    /**
     * Не сохранять аттрибуты. Только для `self::EVENT_UPDATE`
     * @var array<string>
     */
    public array $skipAttributes = [];

    /**
     * @var array<string> List of custom attributes which which are a pair of `key`=>`value` where `key` is attribute name and
     * `value` it anonymous callback function of attribute. Function will be apply for old and value information data.
     * Example:
     * ```php
     * [
     *      'attribute_1' => function($event, $isNewValue) {
     *          if ($isNewValue) {
     *              return $event->sender->attribute_1;
     *          }
     *          return $event->changedAttributes['attribute_1'];
     *      },
     * ]
     * ```
     *  Только для `self::EVENT_UPDATE`.
     */
    public array $customAttributes = [];

    /**
     * Адаптер для AR
     * @var array<string>
     */
    protected array $eventMap = [
        self::EVENT_INSERT => BaseActiveRecord::EVENT_AFTER_INSERT,
        self::EVENT_UPDATE => BaseActiveRecord::EVENT_AFTER_UPDATE,
        self::EVENT_DELETE => BaseActiveRecord::EVENT_AFTER_DELETE,
    ];

    protected ?HistoryModule $module;

    public function init()
    {
        parent::init();

        $this->module = HistoryModule::getInstance();
        $this->skipAttributes = array_fill_keys($this->skipAttributes, true);
    }

    public function events(): array
    {
        $events = [];
        foreach ($this->allowEvents as $name) {
            $events[$this->eventMap[$name]] = 'saveHistory';
        }

        return $events;
    }

    /**
     * Сохранение в хранилище.
     * @throws NotSupportedException
     */
    public function saveHistory(Event $event)
    {
        $rowId = $this->getRowId();
        $tableName = $this->getTableName();
        $modelName = $this->getModelName();
        $createdBy = $this->getCreatedBy();
        $createdAt = time();

        $storage = new $this->module->storage();
        
        switch ($event->name) {
            case BaseActiveRecord::EVENT_AFTER_INSERT:
                $model = new HistoryEntity([
                    'tableName' => $tableName,
                    'modelName' => $modelName,
                    'rowId' => $rowId,
                    'event' => HistoryEntity::EVENT_INSERT,
                    'createdAt' => $createdAt,
                    'createdBy' => $createdBy,
                ]);
                $storage->add($model);
                break;
            
            case BaseActiveRecord::EVENT_AFTER_DELETE:
                $model = new HistoryEntity([
                    'tableName' => $tableName,
                    'modelName' => $modelName,
                    'rowId' => $rowId,
                    'event' => HistoryEntity::EVENT_DELETE,
                    'createdAt' => $createdAt,
                    'createdBy' => $createdBy,
                ]);
                $storage->add($model);
                break;
            
            case BaseActiveRecord::EVENT_AFTER_UPDATE:
                if (!empty($event->changedAttributes)) {
                    foreach ($event->changedAttributes as $name => $value) {
                        if (isset($this->skipAttributes[$name]) || $this->isEquals($name, $value)) {
                            continue;
                        }
                        if (isset($this->customAttributes[$name])) {
                            $oldValue = call_user_func($this->customAttributes[$name], $event, false);
                            $newValue = call_user_func($this->customAttributes[$name], $event, true);
                        } else {
                            $oldValue = $value;
                            $newValue = $this->owner->$name;
                        }
                        $model = new HistoryEntity([
                            'tableName' => $tableName,
                            'modelName' => $modelName,
                            'rowId' => $rowId,
                            'fieldName' => $name,
                            'oldValue' => $oldValue,
                            'newValue' => $newValue,
                            'event' => HistoryEntity::EVENT_UPDATE,
                            'createdAt' => $createdAt,
                            'createdBy' => $createdBy,
                        ]);
                        $storage->add($model);
                    }
                }
                break;
        }

        $storage->flush();
    }
    
    /** @throws NotSupportedException */
    protected function getPrimaryKey(): string
    {
        $primaryKey = $this->owner->primaryKey();
        if (count($primaryKey) == 1) {
            $primaryKey = array_shift($primaryKey);
        } else {
            throw new NotSupportedException('Составной ключ не допустим.');
        }

        return $primaryKey;
    }
    
    /**
     * Получить имя таблицы сущности.
     */
    protected function getTableName(): string
    {
        $owner = $this->owner;

        return $owner::tableName();
    }

    /**
     * Получить имя модели сущности.
     */
    protected function getModelName(): string
    {
        $owner = $this->owner;

        return (new ReflectionClass($owner))->name;
    }
    
    /**
     * Получить пользователя сделавшего изменения.
     */
    protected function getCreatedBy(): int
    {
        return $this->module->user->id;
    }

    /**
     * Возвратить ID строки где были сделаны изменения.
     * @throws NotSupportedException
     */
    protected function getRowId(): int
    {
        $primaryKey = $this->getPrimaryKey();

        return $this->owner->$primaryKey;
    }

    /** @param mixed $value */
    protected function isEquals(string $name, $value): bool
    {
        if (is_object($value)) {
            return $value === $this->owner->$name;
        }

        return $value == $this->owner->$name;
    }
}
