<?php

namespace common\services\auth;

use DomainException;
use RuntimeException;
use Yii;
use common\forms\auth\PasswordResetRequestForm;
use common\forms\auth\ResetPasswordForm;
use common\repositories\UserRepository;
use yii\base\Exception;
use yii\mail\MailerInterface;

class PasswordResetService
{
    private MailerInterface $mailer;
    private UserRepository $users;

    public function __construct(UserRepository $users, MailerInterface $mailer)
    {
        $this->mailer = $mailer;
        $this->users = $users;
    }

    /**
     * Запрос на сброс пароля.
     * @throws Exception
     */
    public function request(PasswordResetRequestForm $form): void
    {
        $user = $this->users->getByEmail($form->email);

        if (!$user->isActive()) {
            throw new DomainException('Пользователь не активирован.');
        }

        $user->requestPasswordReset();
        $this->users->save($user);

        $sent = $this->mailer
            ->compose(
                ['html' => 'auth/reset/confirm-html', 'text' => 'auth/reset/confirm-text'],
                ['user' => $user]
            )
            ->setTo($user->email)
            ->setSubject('Сброс пароля на сайте ' . Yii::$app->name)
            ->send();

        if (!$sent) {
            throw new RuntimeException('Ошибка при отправке.');
        }
    }

    /**
     * Проверка токена сброса пароля
     */
    public function validateToken($token): void
    {
        if (empty($token) || !is_string($token)) {
            throw new DomainException('Токен сброса пароля не может быть пустым.');
        }
        if (!$this->users->existsByPasswordResetToken($token)) {
            throw new DomainException('Неправильный токен сброса пароля.');
        }
    }

    /**
     * Сброс пароля
     * @throws Exception
     */
    public function reset(string $token, ResetPasswordForm $form): void
    {
        $user = $this->users->getByPasswordResetToken($token);
        $user->resetPassword($form->password);
        $this->users->save($user);
    }
}
