<?php

namespace common\tests\unit\modules;

use common\modules\entitiesHistory\behaviors\HistoryBehavior;
use common\modules\entitiesHistory\entities\History;
use common\tests\unit\modules\models\User;
use common\tests\unit\modules\models\Post;
use Yii;
use yii\db\Query;

class HistoryTest extends TestCase
{
    public function testInsertEventRecord()
    {
        $post = new Post();

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
        ]);

        $post->title = 'Новый заголовок';
        $post->content = 'Новый контент';
        $post->type = Post::TYPE_NEWS;
        $post->created_at = time();
        $post->updated_at = time();

        $this->assertTrue($post->save());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->one();

        $this->assertNotFalse($history);
        $this->assertEquals(1, $history['event']);
        $this->assertEquals('{{%post}}', $history['table_name']);
        $this->assertEquals(100, $history['created_by']);
        $this->assertNotNull($history['created_at']);
    }

    public function testUpdateEventRecord()
    {
        $post = Post::findOne(1);

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
        ]);

        $post->title = 'Изменить заголовок';
        $post->content = 'Изменить контент';

        $this->assertTrue($post->save());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->indexBy('field_name')
            ->all();

        $this->assertCount(2, $history);
        $this->assertEquals(2, $history['title']['event']);
        $this->assertEquals('Пример заголовка 1', $history['title']['old_value']);
        $this->assertEquals('Изменить заголовок', $history['title']['new_value']);
        $this->assertEquals(2, $history['content']['event']);
        $this->assertEquals('Пример контента 1', $history['content']['old_value']);
        $this->assertEquals('Изменить контент', $history['content']['new_value']);
        $this->assertFalse(isset($history['type']));
    }

    public function testDeleteEventRecord()
    {
        $post = Post::findOne(1);

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
        ]);

        $this->assertGreaterThan(0, $post->delete());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->one();

        $this->assertNotFalse($history);
        $this->assertEquals(3, $history['event']);
        $this->assertNull($history['field_name']);
        $this->assertNull($history['old_value']);
        $this->assertNull($history['new_value']);
        $this->assertEquals('{{%post}}', $history['table_name']);
        $this->assertEquals(100, $history['created_by']);
        $this->assertNotNull($history['created_at']);
    }

    public function testCustomAttributes()
    {
        $post = Post::findOne(1);

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
            'customAttributes' => [
                'type' => function ($event, $isNewValue) {
                    if ($isNewValue) {
                        return Post::getTypes()[$event->sender->type];
                    }
                    return Post::getTypes()[$event->changedAttributes['type']];
                },
            ],
        ]);

        $post->title = 'Изменить заголовок';
        $post->type = Post::TYPE_ARTICLE;

        $this->assertTrue($post->save());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->indexBy('field_name')
            ->all();

        $this->assertCount(2, $history);
        $this->assertEquals('Изменить заголовок', $history['title']['new_value']);
        $this->assertEquals('Новости', $history['type']['old_value']);
        $this->assertEquals('Статья', $history['type']['new_value']);
    }

    public function testSkipAttributes()
    {
        $post = Post::findOne(1);

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
            'skipAttributes' => [
                'type',
            ],
        ]);

        $post->title = 'Изменить заголовок';
        $post->type = Post::TYPE_ARTICLE;

        $this->assertTrue($post->save());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->indexBy('field_name')
            ->all();

        $this->assertEquals('Изменить заголовок', $history['title']['new_value']);
        $this->assertFalse(isset($history['type']));
    }

    public function testIgnoreInsertEvent()
    {
        // insert
        $post = new Post();

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
            'allowEvents' => [
                History::EVENT_UPDATE,
                History::EVENT_DELETE,
            ],
        ]);

        $post->title = 'Новый заголовок';
        $post->content = 'Новый контент';
        $post->type = Post::TYPE_NEWS;
        $post->created_at = time();
        $post->updated_at = time();

        $this->assertTrue($post->save());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->one();

        $this->assertFalse($history);
    }

    public function testIgnoreUpdateEvent()
    {
        $post = Post::findOne(1);

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
            'allowEvents' => [
                History::EVENT_INSERT,
                History::EVENT_DELETE,
            ],
        ]);

        $post->title = 'Изменить заголовок';

        $this->assertTrue($post->save());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->indexBy('field_name')
            ->all();

        $this->assertCount(0, $history);
    }

    public function testIgnoreDeleteEvent()
    {
        $post = Post::findOne(1);

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
            'allowEvents' => [
                History::EVENT_INSERT,
                History::EVENT_UPDATE,
            ],
        ]);

        $this->assertGreaterThan(0, $post->delete());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->one();

        $this->assertFalse($history);
    }

    public function testSkipAttributesWithDbExpression()
    {
        // test 1
        $post = Post::findOne(2);

        $post->attachBehavior('history', [
            'class' => HistoryBehavior::class,
            'skipAttributes' => [
                'updated_at',
            ],
        ]);

        $post->title = 'Изменить заголовок для пропуска атрибута';
        $post->updated_at = time();

        $this->assertTrue($post->save());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->indexBy('field_name')
            ->all();

        $this->assertFalse(isset($history['updated_at']));

        // test 2
        $post->updated_at = time();

        $this->assertTrue($post->save());

        $q = new Query();
        $history = $q->from('{{%model_histories}}')
            ->where(['row_id' => $post->id])
            ->indexBy('field_name')
            ->all();

        $this->assertFalse(isset($history['updated_at']));
    }

    protected function setUp(): void
    {
        parent::setUp();

        Yii::$app->user->login(User::findIdentity(100));
    }
}
